class LifePex::Pex < Sequel::Model(:pexs)
  one_to_many :user_pexs
  many_to_one :user

  # common interface for setup_user_pexs_*
  def self.setup_user_pexs(user_id: nil, user_pexs: nil)
    if user_id
      setup_user_pexs_by_user_id(user_id)
    elsif user_pexs
      setup_user_pexs_by_user_pexs(user_pexs)
    else
      raise "Invalid setup"
    end
  end

  def self.setup_user_pexs_by_user_id(user_id)
    user_pexs = LifePex::UserPex.where(user_id: user_id)
    setup_user_pexs_by_user_pexs(user_pexs)
  end

  def self.setup_user_pexs_by_user_pexs(user_pexs)
    pexs = LifePex::Pex.order(:name).all()
             .group_by { |pex| pex[:id] }
             .map { |id, pex_group| [id, pex_group[0]] }
             .to_h
    pexs.each do |id, pex|
      pex[:count_by_date] = {}
    end

    user_pexs.each do |user_pex|
      pex_id = user_pex[:pex_id]
      pex = pexs[pex_id]
      pex[:count_by_date][user_pex[:created_at]] ||= 0
      pex[:count_by_date][user_pex[:created_at]] += 1
    end

    pexs.each do |id, pex|
      pex[:total_by_date] = pex[:count_by_date].map { |date, count| [date, count * pex[:amount]] }.to_h
    end

    pexs
  end

  PUBLIC_FIELDS = %w(id name category amount)

  def to_s
    fields = PUBLIC_FIELDS.map { |f| "#{f} #{self.send(f)}" }.join(' ')
    "Pex(#{fields})"
  end

  # TODO what the hell is params, is it an io or someting ?
  def to_json(*params)
    PUBLIC_FIELDS.map { |f| [f, self.send(f)] }.to_h.to_json
  end
end
