require "yaml"
require_relative "./json_api.rb"

class UserSystem < AuthSystem
  # use AuthSystem
  include JSON::API

  DEFAULT_PEXS_FOR_NEW_USERS = YAML.load_file "config/default_pexs_for_new_users.yaml"

  get "/login" do
    slim :login
  end

  def login(params)
    user = LifePex::User.where(username: params["username"]).first
    if user && BCrypt::Password.new(user[:hashed_password]) == params["password"]
      cookies["auth"] = JWT.encode({ "user_id" => user[:id] }, LifePex::SECRET)
      user
    else
      nil
    end
  end

  post "/login", provides: 'html' do
    if user = login(params)
      cookies["auth"] = JWT.encode({ "user_id" => user[:id] }, LifePex::SECRET)
      redirect "/"
    else
      slim :login, locals: { flash: { danger: "Failed to login" } }
    end
  end

  get "/register" do
    slim :register
  end

  def register(params)
    user = LifePex::User.where(username: params["username"]).first
    if !user
      hashed_password = BCrypt::Password.create params["password"]
      user_id = LifePex::User.insert(username: params["username"], hashed_password: hashed_password)
      cookies["auth"] = JWT.encode({ "user_id" => user_id }, LifePex::SECRET)
      DEFAULT_PEXS_FOR_NEW_USERS.each do |pex|
        LifePex::Pex.insert(**pex, user_id: user_id)
      end
      user_id
    else
      nil
    end
  end

  post "/register", provides: 'html' do
    if user_id = register(params)
      redirect "/"
    else
      slim :register, locals: { flash: { danger: "Failed to register as #{params['username']}" } }
    end
  end

  get "/logout", auth: [] do
    cookies.delete "auth"
    slim :logout, locals: { flash: { success: "Logged out" } }
  end

  get "/password", auth: [] do
    slim :password
  end

  def change_password(params)
    hashed_password = BCrypt::Password.create params["password"]
    current_user.update(hashed_password: hashed_password)
  end

  post "/password", auth: [], provides: 'html' do
    if params["password"] != params["password_confirmation"]
      slim :password, locals: { flash: { warning: "Password don't match" } }
    else
      change_password(params)
      slim :password, locals: { flash: { success: "Password updated" } }
    end
  end

  register Sinatra::Namespace
  namespace '/api/users/v1' do
    post "/login", provides: 'json' do
      if user = login(json_params)
        { message: 'Logged in' }.to_json
      else
        halt 400, { message: 'Invalid credentials' }.to_json
      end
    end

    post "/register", provides: 'json' do
      if user_id = register(json_params)
        { message: "Registered as user_id=#{user_id}", user: { id: user_id, username: json_params["username"] } }.to_json
      else
        halt 400, { message: "Failed to register as #{json_params['username']}" }.to_json
      end
    end

    post "/password", auth: [], provides: 'html' do
      change_password(json_params)
      { message: 'password changed' }.to_json
    end
  end
end
